package cn.com.qmth.stmms.biz.report.dao;

import java.util.List;

import org.springframework.data.jpa.repository.JpaSpecificationExecutor;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.PagingAndSortingRepository;

import cn.com.qmth.stmms.biz.report.model.ReportSubjectTeacher;

/**
 * 课程老师成绩分析 Dao
 *
 * @author xuwenjin
 * @date 2019-04-16
 */
public interface ReportSubjectTeacherDao extends PagingAndSortingRepository<ReportSubjectTeacher, Integer>,
        JpaSpecificationExecutor<ReportSubjectTeacher> {

    @Modifying
    @Query("delete from ReportSubjectTeacher r where r.examId=?1 ")
    void deleteByExamId(Integer exmaId);

    /**
     * 获取课程老师分析数据
     *
     * @param examId
     *            考试Id
     * @param subjectCode
     *            课程编码
     */
    @Query(value = "SELECT " + "s.teacher," + "count(s) - sum( CASE WHEN s.absent = true THEN 1 ELSE 0 END ) ,"
            + "sum( CASE WHEN ( s.objectiveScore + s.subjectiveScore ) >= e.passScore THEN 1 ELSE 0 END ) ,"
            + "sum( CASE WHEN ( s.objectiveScore + s.subjectiveScore ) >= e.excellentScore THEN 1 ELSE 0 END ) ,"
            + "max( s.objectiveScore + s.subjectiveScore ) ," + "min( s.objectiveScore + s.subjectiveScore ) ,"
            + "avg( s.objectiveScore + s.subjectiveScore ) " + "FROM ExamStudent s , Exam e "
            + "where s.examId = e.id " + "and s.examId = ?1 " + "AND s.subjectCode = ?2  and s.teacher is not null "
            + "GROUP BY s.teacher")
    List<Object[]> queryReportSubjectTeacherData(Integer examId, String subjectCode);

    @Modifying
    @Query("update ReportSubjectTeacher r set r.relativeAvgScore = (r.avgScore - (select s.avgScore from ReportSubject s where s.subjectCode = r.subjectCode and s.examId = r.examId )) where r.examId=?1  ")
    void updateRelativeAvgScore(Integer examId);

}
